% this is an matlab wrapper to run the ranking svm of Thorsten Joachims,
% which is available at: http://www.cs.cornell.edu/people/tj/svm_light/svm_rank.html
% svm_rank_linux64 library is needed
% written by dengxin dai, CVL, ETH Zurich, 
% dai@vision.ee.ethz.ch 

%% data simulation 
% data format:
% RD.fet [N, F]: each row is a feature vector of one sample, float or double
% RD.gid [N, 1]: group ids of all samples, integers, each sample only compared with samples sharing the same group id.  
% RD.pref: preference of samples, float or double   
% note: make sure that each group has at least 2 samples, for the sake of comparison

clear all; clc;

% simulate data for ranking function y = 2xf1 + 3xf2 + 8xf3
W = [2 5 8]; 
N = 200;            
group_size = 4;  % the size of each group, larger than 2

% training data
RD_tr.fet = rand([N, numel(W)]);
RD_tr.pref = sum(RD_tr.fet.*repmat(W, [N, 1]), 2);
RD_tr.gid = floor(([1:N-1 N-1])/group_size)';   % make sure that each group has at least 2 samples, for the sake of comparison

% test data 
N = 100;
RD_te.fet = rand([N, numel(W)]);
RD_te.pref = sum(RD_te.fet.*repmat(W, [N, 1]), 2); % only used for evaluation
RD_te.gid = floor([(1:N-1) N-1]/group_size)';      % only used for evaluation, values don't affect ranking results 

%% absolute path of ranking svm, change this to the path where you store the ranking svm, which is downloadable at http://www.cs.cornell.edu/people/tj/svm_light/svm_rank.html

ranksvmPath = [pwd '/svm_rank_linux64/'];
% '/scratch_net/pengyou/Dai/Code/matlab_lib/lib_daid/ranking/svm_rank_linux64/'; 

%%  svm_ranking training & testing

svm_ranking_training(ranksvmPath, RD_tr);

test_predictions = svm_ranking_test(ranksvmPath, RD_te);

%% evaluation by comparing ranking results to gt

gids_te = unique(RD_te.gid);

for i=1:numel(gids_te)
    i_ind = find(RD_te.gid==gids_te(i));
    [~, predict_order] = sort(test_predictions(i_ind));
    [~, gt_order] = sort(RD_te.pref(i_ind));
    rank_score(i, 1) = corr(predict_order, gt_order);
end

fprintf('ranking score is: %f \n', mean(rank_score));


